

/*Corse dei Cavalli*/

import java.io.*;
import java.net.*;
import java.util.*;

class BetServer{

	
    private HashMap scommesse;
    private Calendar limite;
    private BetAccepter accepter;
    private BetDenyer   denyer;
    private int port;


    public BetServer(int port, Calendar deadline){
	
	scommesse=new HashMap();
	limite=deadline;
        this.port=port;
	accepter=new BetAccepter(port);
	accepter.start();
    }



    class BetAccepter extends Thread{

	private int port;
	private ServerSocket serv;
        private boolean accept;

	BetAccepter (int p){
	   try{
	     port=p;
	     serv=new ServerSocket(port);
	     accept=true;
           } catch (IOException ioe){
		System.out.println(ioe);}
	}
	
	public void run(){

	   while(accept){
		try{
	   	   Calendar now=Calendar.getInstance();   
		   serv.setSoTimeout((int)(limite.getTimeInMillis()-now.getTimeInMillis()));   
		   Socket k=serv.accept();
	   	   BufferedReader in = new BufferedReader (new InputStreamReader (k.getInputStream()));
	   	   PrintWriter out = new PrintWriter (k.getOutputStream(), true );
	   
	   	   String line=in.readLine();
	       	   int pos=line.indexOf(" ");
	       	   int numCavallo=Integer.parseInt(line.substring(0,pos));
	       	   long puntata=Long.parseLong(line.substring(pos+1));
	       	   InetAddress ip=k.getInetAddress();
		
                   Scommessa s=new Scommessa(numCavallo,puntata,ip);
	       	   int key=s.getID();
	       	   scommesse.put(new Integer(key),s);

		   //manda ok
		   out.println("Scommessa accettata.");
		   out.close();
		   k.close();
		   System.out.println("Ricevuta scommessa "+ip+" "+numCavallo+" "+puntata);
                 
	         } catch (SocketTimeoutException ste){         
			  accept=false;
			  System.out.println("Tempo a disposizione per le scommesse terminato");
	           }// catch
                   catch (IOException ioe){
		        System.out.println(ioe);}
	    }//while
	    try{
	      serv.close();
	    } catch (IOException ioe){
		        System.out.println(ioe);}
	}// run
    }// BetAccepter


    class BetDenyer extends Thread{

	private int port;
	private ServerSocket serv;
        private boolean closed;

	BetDenyer (int p){
	   try{
	     port=p;
	     serv=new ServerSocket(port);
	     closed=true;
	   } catch (IOException ioe){
		System.out.println(ioe);}
	}

	public void reset(){
	   closed=false;
	}
	
	public void run(){
           try{
    	     while(closed){
	   	Socket k=serv.accept();
	   	PrintWriter out = new PrintWriter(k.getOutputStream(),true);
                out.println("Scommesse chiuse");
		out.close();
	   	k.close();
		System.out.println("Rifiutata scommessa");
	     }//while
	     serv.close();	   
	    } catch (IOException ioe){
		        System.out.println(ioe);} 
	}// run	
    }// BetDenyer


    public void attendiChiusuraScommesse(){
	try{
	   accepter.join();
        } catch (InterruptedException ie){
		        System.out.println(ie);}
    }

    public void chiusuraScommesse(){
	denyer=new BetDenyer(port);
	denyer.start();
    }

    public void resetServer(){
	denyer.reset();
    }

    public LinkedList controllaScommesse (int cavalloVincente){
	
	LinkedList elenco=new LinkedList();
	for (int i=0;i<=scommesse.size();i++){
	    Scommessa s=(Scommessa)scommesse.get(new Integer(i));
	    if (s.equals(new Scommessa(cavalloVincente,0,null)))
		elenco.addLast(s);
	}
	return elenco;
    }

    public void comunicaVincitori(LinkedList vincitori, InetAddress ind, int port){
	
	ListIterator it=vincitori.listIterator();
	try{
	   MulticastSocket socket=new MulticastSocket();
	   byte[] buf=new byte[256];
	   String m="";
	   while(it.hasNext()){
	       Scommessa s=(Scommessa)it.next();
	       m+=s.getCavallo()+" "+(s.getPuntata()*12)+"\n";
	   }
	   buf=m.getBytes();
	   DatagramPacket pk=new DatagramPacket(buf,buf.length,ind,port);
	   socket.send(pk);
        } catch (IOException ioe){
		        System.out.println(ioe);}
    }

    
    public static void main (String[] args){
	
	int serverPort=8001;
	int clientPort=8002;
	try{
	   InetAddress multiAddress=InetAddress.getByName("230.0.0.1");
        
	   LinkedList elenco;
	
	   Calendar deadline=Calendar.getInstance();	
	   deadline.add(Calendar.MINUTE,1);

	   BetServer server=new BetServer(serverPort, deadline);
	   server.attendiChiusuraScommesse();
	   server.chiusuraScommesse();
           int vincente=((int)(Math.random()*12))+1;
	   System.out.println("E' risultato vincitore il cavallo "+vincente);
	   elenco=server.controllaScommesse(vincente);
	   server.comunicaVincitori(elenco,multiAddress,clientPort);
	   System.out.println("Elenco vincitori:");	   
	   System.out.println(elenco);
	
           Thread.sleep(150000);
           server.resetServer();
	 } catch (InterruptedException ie){
		        System.out.println(ie);}
	   catch (UnknownHostException uhe){
		        System.out.println(uhe);}
	
     }// main

}// BetServer


